#!/usr/bin/env gjs

const GLib = imports.gi.GLib;
const Gio = imports.gi.Gio;
const Gtk = imports.gi.Gtk;
const WebKit = imports.gi.WebKit;

const Lang = imports.lang;
const Mainloop = imports.mainloop;
const JsUnit = imports.jsUnit;

var TOP_SRCDIR = '';

var headless = true; // It require X11
var passTest = false;
var slowDownFactor = 1;

function Backend() {
    this._init();
}

Backend.prototype = {
    _init: function () {
        Gtk.init(null, null);

        this._scrolledWindow = new Gtk.ScrolledWindow();

        if (!headless) {
            this._window = new Gtk.Window({ type: Gtk.WindowType.TOPLEVEL, decorated: true });
            this._window.set_position(Gtk.WindowPosition.CENTER);
            this._window.connect('destroy', function() {
                Gtk.main_quit();
            });

            this._scrolledWindow.set_size_request(1024, 800);
            this._window.add(this._scrolledWindow);

            this._window.show_all();
        }
        this._webview = new WebKit.WebView({ visible: true });
        let settings = this._webview.get_settings();
        settings.enable_plugins = false;
        settings.html5_local_storage_database_path = '/tmp/webapps_' + Math.random(); // FIXME: g_file_new_tmp didn't work
        settings.enable_xss_auditor = false;

        this._scrolledWindow.add(this._webview);
    },

    connect: function(eventName, callback) {
        if (eventName == "load") {
            this._webview.connect('notify::load-status', function(view, frame) {
                if (this._webview.load_status == WebKit.LoadStatus.FINISHED
                    || this._webview.load_status == WebKit.LoadStatus.FAILED) {
                    callback();
                }
            }.bind(this));
        } else if (eventName == "unload") {
            this._webview.connect('navigation-policy-decision-requested', callback);
        } else {
            throw new Error();
        }
    },

    getContent: function(path) {
        if (path.match(/^\//)) {
            let [res, content] = GLib.file_get_contents(path);
            return String(content);
        }

        let [res, content] = GLib.file_get_contents(TOP_SRCDIR + path);
        return String(content);
    },

    loadURI: function(uri) {
        this._webview.load_uri(uri);
    },

    assertEquals: function() {
        return JsUnit.assertEquals.apply(null, arguments);
    },

    injectScript: function(src) {
        this._webview.execute_script('(function() { ' + String(src) + '})();');
    },

    getLocation: function() {
        return String(this._webview.get_uri());
    },

    getTextFromNode: function(nodeName) {
        return this._webview.get_dom_document().get_element_by_id(nodeName).get_text_content();
    },

    print: function(str) {
        print(str);
    },

    finish: function() {
        Gtk.main_quit();
    },

    setTimeout: function(callback, ms) {
        Mainloop.timeout_add(ms * slowDownFactor, function() {
            callback();
            return false;
        });
    }
};

if (ARGV.length && ARGV[0] == '--show-window') {
    headless = false;
    ARGV.shift();
}

if (ARGV.length > 2 && ARGV[0] == '--slow-down-factor') {
    slowDownFactor = Number(ARGV[1]);
    if (!slowDownFactor) {
        printerr('invalid slowDownFactor');
        slowDownFactor = 1;
    }
    ARGV.shift();
    ARGV.shift();
}

if (ARGV.length == 1) {
    let file = new Gio.file_new_for_path(ARGV[0]);

    TOP_SRCDIR = String(file.get_parent().get_parent().get_path() + '/');

    [res, content] = GLib.file_get_contents(TOP_SRCDIR + "scripts/testSuiteApi.js");
    eval(String(content));

    new TestSuiteAPI(new Backend(), file.get_path());
    Gtk.main();
} else {
    printerr('Usage: test.js pathToTestScript');
}
